package java.util;

import java.text.NumberFormat;
import java.text.DecimalFormat;
import java.time.DateTimeException;
import java.time.Instant;
import java.time.ZoneOffset;
import java.time.temporal.TemporalQueries;
import java.time.ZoneId;
import java.time.temporal.ChronoField;
import java.text.DateFormatSymbols;
import java.math.RoundingMode;
import java.math.MathContext;
import sun.misc.FormattedFloatingDecimal;
import java.time.temporal.TemporalAccessor;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.regex.Matcher;
import java.text.DecimalFormatSymbols;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.FileNotFoundException;
import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.File;
import java.nio.charset.UnsupportedCharsetException;
import java.nio.charset.IllegalCharsetNameException;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.util.regex.Pattern;
import java.io.IOException;
import java.io.Flushable;
import java.io.Closeable;

public final class Formatter implements Closeable, Flushable
{
    private Appendable a;
    private final Locale l;
    private IOException lastException;
    private final char zero;
    private static double scaleUp;
    private static final int MAX_FD_CHARS = 30;
    private static final String formatSpecifier = "%(\\d+\\$)?([-#+ 0,(\\<]*)?(\\d+)?(\\.\\d+)?([tT])?([a-zA-Z%])";
    private static Pattern fsPattern;
    
    private static Charset toCharset(String s) throws UnsupportedEncodingException {
        Objects.requireNonNull(s, "charsetName");
        try {
            return Charset.forName(s);
        }
        catch (IllegalCharsetNameException | UnsupportedCharsetException ex) {
            throw new UnsupportedEncodingException(s);
        }
    }
    
    private static final Appendable nonNullAppendable(Appendable appendable) {
        if (appendable == null) {
            return new StringBuilder();
        }
        return appendable;
    }
    
    private Formatter(Locale l, Appendable a) {
        this.a = a;
        this.l = l;
        this.zero = getZero(l);
    }
    
    private Formatter(Charset cs, Locale locale, File file) throws FileNotFoundException {
        this(locale, new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), cs)));
    }
    
    public Formatter() {
        this(Locale.getDefault(Locale.Category.FORMAT), new StringBuilder());
    }
    
    public Formatter(Appendable appendable) {
        this(Locale.getDefault(Locale.Category.FORMAT), nonNullAppendable(appendable));
    }
    
    public Formatter(Locale locale) {
        this(locale, new StringBuilder());
    }
    
    public Formatter(Appendable appendable, Locale locale) {
        this(locale, nonNullAppendable(appendable));
    }
    
    public Formatter(String name) throws FileNotFoundException {
        this(Locale.getDefault(Locale.Category.FORMAT), new BufferedWriter(new OutputStreamWriter(new FileOutputStream(name))));
    }
    
    public Formatter(String s, String s2) throws FileNotFoundException, UnsupportedEncodingException {
        this(s, s2, Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public Formatter(String pathname, String s, Locale locale) throws FileNotFoundException, UnsupportedEncodingException {
        this(toCharset(s), locale, new File(pathname));
    }
    
    public Formatter(File file) throws FileNotFoundException {
        this(Locale.getDefault(Locale.Category.FORMAT), new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file))));
    }
    
    public Formatter(File file, String s) throws FileNotFoundException, UnsupportedEncodingException {
        this(file, s, Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public Formatter(File file, String s, Locale locale) throws FileNotFoundException, UnsupportedEncodingException {
        this(toCharset(s), locale, file);
    }
    
    public Formatter(PrintStream printStream) {
        this(Locale.getDefault(Locale.Category.FORMAT), (Appendable)Objects.requireNonNull(printStream));
    }
    
    public Formatter(OutputStream out) {
        this(Locale.getDefault(Locale.Category.FORMAT), new BufferedWriter(new OutputStreamWriter(out)));
    }
    
    public Formatter(OutputStream outputStream, String s) throws UnsupportedEncodingException {
        this(outputStream, s, Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public Formatter(OutputStream out, String charsetName, Locale locale) throws UnsupportedEncodingException {
        this(locale, new BufferedWriter(new OutputStreamWriter(out, charsetName)));
    }
    
    private static char getZero(Locale locale) {
        if (locale != null && !locale.equals(Locale.US)) {
            return DecimalFormatSymbols.getInstance(locale).getZeroDigit();
        }
        return '0';
    }
    
    public Locale locale() {
        this.ensureOpen();
        return this.l;
    }
    
    public Appendable out() {
        this.ensureOpen();
        return this.a;
    }
    
    @Override
    public String toString() {
        this.ensureOpen();
        return this.a.toString();
    }
    
    @Override
    public void flush() {
        this.ensureOpen();
        if ((this.a instanceof Flushable)) {
            try {
                ((Flushable)this.a).flush();
            }
            catch (IOException lastException) {
                this.lastException = lastException;
            }
        }
    }
    
    @Override
    public void close() {
        if (this.a == null) {
            return;
        }
        try {
            if ((this.a instanceof Closeable)) {
                ((Closeable)this.a).close();
            }
        }
        catch (IOException lastException) {
            this.lastException = lastException;
        }
        finally {
            this.a = null;
        }
        return;
    }
    
    private void ensureOpen() {
        if (this.a == null) {
            throw new FormatterClosedException();
        }
        return;
    }
    
    public IOException ioException() {
        return this.lastException;
    }
    
    public Formatter format(String s, Object... array) {
        return this.format(this.l, s, array);
    }
    
    public Formatter format(Locale locale, String s, Object... array) {
        this.ensureOpen();
        int n = -1;
        int n2 = -1;
        Formatter.FormatString[] parse = this.parse(s);
        for (int i = 0; i < parse.length; ++i) {
            Formatter.FormatString formatString = parse[i];
            int index = formatString.index();
            try {
                switch (index) {
                    case -2: {
                        formatString.print(null, locale);
                        break;
                    }
                    case -1: {
                        if (n < 0 || (array != null && n > array.length - 1)) {
                            throw new MissingFormatArgumentException(formatString.toString());
                        }
                        formatString.print((array == null) ? null : array[n], locale);
                        break;
                    }
                    case 0: {
                        n = ++n2;
                        if (array != null && n2 > array.length - 1) {
                            throw new MissingFormatArgumentException(formatString.toString());
                        }
                        formatString.print((array == null) ? null : array[n2], locale);
                        break;
                    }
                    default: {
                        n = index - 1;
                        if (array != null && n > array.length - 1) {
                            throw new MissingFormatArgumentException(formatString.toString());
                        }
                        formatString.print((array == null) ? null : array[n], locale);
                        break;
                    }
                }
            }
            catch (IOException lastException) {
                this.lastException = lastException;
            }
        }
        return this;
    }
    
    private Formatter.FormatString[] parse(String input) {
        ArrayList list = new ArrayList();
        Matcher matcher = Formatter.fsPattern.matcher(input);
        int i = 0;
        int length = input.length();
        while (i < length) {
            if (matcher.find(i)) {
                if (matcher.start() != i) {
                    checkText(input, i, matcher.start());
                    list.add(new Formatter.FixedString(input.substring(i, matcher.start())));
                }
                list.add(new Formatter.FormatSpecifier(matcher));
                i = matcher.end();
            }
            else {
                checkText(input, i, length);
                list.add(new Formatter.FixedString(input.substring(i)));
                break;
            }
        }
        return (Formatter.FormatString[])list.toArray(new Formatter.FormatString[list.size()]);
    }
    
    private static void checkText(String s, int i, int n) {
        while (i < n) {
            if (s.charAt(i) == '%') {
                throw new UnknownFormatConversionException(String.valueOf((i == n - 1) ? '%' : s.charAt(i + 1)));
            }
            ++i;
        }
    }
    
    static {
        Formatter.fsPattern = Pattern.compile("%(\\d+\\$)?([-#+ 0,(\\<]*)?(\\d+)?(\\.\\d+)?([tT])?([a-zA-Z%])");
    }
    
    private class FixedString implements Formatter.FormatString
    {
        private String s;
        
        FixedString(String s) {
            this.s = s;
        }
        
        @Override
        public int index() {
            return -2;
        }
        
        @Override
        public void print(Object o, Locale locale) throws IOException {
            Formatter.this.a.append(this.s);
        }
        
        @Override
        public String toString() {
            return this.s;
        }
    }
    
    public enum BigDecimalLayoutForm
    {
        SCIENTIFIC, 
        DECIMAL_FLOAT;
    }
    
    private class FormatSpecifier implements Formatter.FormatString
    {
        private int index;
        private Formatter.Flags f;
        private int width;
        private int precision;
        private boolean dt;
        private char c;
        static final /* synthetic */ boolean $assertionsDisabled;
        
        private int index(String s) {
            if (s != null) {
                try {
                    this.index = Integer.parseInt(s.substring(0, s.length() - 1));
                }
                catch (NumberFormatException ex) {
                    assert false;
                }
            }
            else {
                this.index = 0;
            }
            return this.index;
        }
        
        @Override
        public int index() {
            return this.index;
        }
        
        private Formatter.Flags flags(String s) {
            this.f = Formatter.Flags.parse(s);
            if (this.f.contains(Formatter.Flags.PREVIOUS)) {
                this.index = -1;
            }
            return this.f;
        }
        
        Formatter.Flags flags() {
            return this.f;
        }
        
        private int width(String s) {
            this.width = -1;
            if (s != null) {
                try {
                    this.width = Integer.parseInt(s);
                    if (this.width < 0) {
                        throw new IllegalFormatWidthException(this.width);
                    }
                }
                catch (NumberFormatException ex) {
                    assert false;
                }
            }
            return this.width;
        }
        
        int width() {
            return this.width;
        }
        
        private int precision(String s) {
            this.precision = -1;
            if (s != null) {
                try {
                    this.precision = Integer.parseInt(s.substring(1));
                    if (this.precision < 0) {
                        throw new IllegalFormatPrecisionException(this.precision);
                    }
                }
                catch (NumberFormatException ex) {
                    assert false;
                }
            }
            return this.precision;
        }
        
        int precision() {
            return this.precision;
        }
        
        private char conversion(String s) {
            this.c = s.charAt(0);
            if (!this.dt) {
                if (!Formatter.Conversion.isValid(this.c)) {
                    throw new UnknownFormatConversionException(String.valueOf(this.c));
                }
                if (Character.isUpperCase(this.c)) {
                    Formatter.Flags.access$100(this.f, Formatter.Flags.UPPERCASE);
                }
                this.c = Character.toLowerCase(this.c);
                if (Formatter.Conversion.isText(this.c)) {
                    this.index = -2;
                }
            }
            return this.c;
        }
        
        private char conversion() {
            return this.c;
        }
        
        FormatSpecifier(Matcher matcher) {
            this.index = -1;
            this.f = Formatter.Flags.NONE;
            this.dt = false;
            int group = 1;
            this.index(matcher.group(group++));
            this.flags(matcher.group(group++));
            this.width(matcher.group(group++));
            this.precision(matcher.group(group++));
            String group2 = matcher.group(group++);
            if (group2 != null) {
                this.dt = true;
                if (group2.equals("T")) {
                    Formatter.Flags.access$100(this.f, Formatter.Flags.UPPERCASE);
                }
            }
            this.conversion(matcher.group(group));
            if (this.dt) {
                this.checkDateTime();
            }
            else if (Formatter.Conversion.isGeneral(this.c)) {
                this.checkGeneral();
            }
            else if (Formatter.Conversion.isCharacter(this.c)) {
                this.checkCharacter();
            }
            else if (Formatter.Conversion.isInteger(this.c)) {
                this.checkInteger();
            }
            else if (Formatter.Conversion.isFloat(this.c)) {
                this.checkFloat();
            }
            else if (Formatter.Conversion.isText(this.c)) {
                this.checkText();
            }
            else {
                throw new UnknownFormatConversionException(String.valueOf(this.c));
            }
        }
        
        @Override
        public void print(Object o, Locale locale) throws IOException {
            if (this.dt) {
                this.printDateTime(o, locale);
                return;
            }
            switch (this.c) {
                case 'd':
                case 'o':
                case 'x': {
                    this.printInteger(o, locale);
                    break;
                }
                case 'a':
                case 'e':
                case 'f':
                case 'g': {
                    this.printFloat(o, locale);
                    break;
                }
                case 'C':
                case 'c': {
                    this.printCharacter(o);
                    break;
                }
                case 'b': {
                    this.printBoolean(o);
                    break;
                }
                case 's': {
                    this.printString(o, locale);
                    break;
                }
                case 'h': {
                    this.printHashCode(o);
                    break;
                }
                case 'n': {
                    Formatter.this.a.append(System.lineSeparator());
                    break;
                }
                case '%': {
                    Formatter.this.a.append('%');
                    break;
                }
                default: {
                    assert false;
                    break;
                }
            }
            return;
        }
        
        private void printInteger(Object o, Locale locale) throws IOException {
            if (o == null) {
                this.print("null");
            }
            else if ((o instanceof Byte)) {
                this.print((byte)o, locale);
            }
            else if ((o instanceof Short)) {
                this.print((short)o, locale);
            }
            else if ((o instanceof Integer)) {
                this.print((int)o, locale);
            }
            else if ((o instanceof Long)) {
                this.print((long)o, locale);
            }
            else if ((o instanceof BigInteger)) {
                this.print((BigInteger)o, locale);
            }
            else {
                this.failConversion(this.c, o);
            }
        }
        
        private void printFloat(Object o, Locale locale) throws IOException {
            if (o == null) {
                this.print("null");
            }
            else if ((o instanceof Float)) {
                this.print((float)o, locale);
            }
            else if ((o instanceof Double)) {
                this.print((double)o, locale);
            }
            else if ((o instanceof BigDecimal)) {
                this.print((BigDecimal)o, locale);
            }
            else {
                this.failConversion(this.c, o);
            }
        }
        
        private void printDateTime(Object o, Locale locale) throws IOException {
            if (o == null) {
                this.print("null");
                return;
            }
            Calendar calendar = null;
            if ((o instanceof Long)) {
                calendar = Calendar.getInstance((locale == null) ? Locale.US : locale);
                calendar.setTimeInMillis((long)o);
            }
            else if ((o instanceof Date)) {
                calendar = Calendar.getInstance((locale == null) ? Locale.US : locale);
                calendar.setTime((Date)o);
            }
            else if ((o instanceof Calendar)) {
                calendar = (Calendar)((Calendar)o).clone();
                calendar.setLenient(true);
            }
            else {
                if ((o instanceof TemporalAccessor)) {
                    this.print((TemporalAccessor)o, this.c, locale);
                    return;
                }
                this.failConversion(this.c, o);
            }
            this.print(calendar, this.c, locale);
            return;
        }
        
        private void printCharacter(Object o) throws IOException {
            if (o == null) {
                this.print("null");
                return;
            }
            String string = null;
            if ((o instanceof Character)) {
                string = ((Character)o).toString();
            }
            else if ((o instanceof Byte)) {
                int byteValue = (byte)o;
                if (Character.isValidCodePoint(byteValue)) {
                    string = new String(Character.toChars(byteValue));
                }
                else {
                    throw new IllegalFormatCodePointException(byteValue);
                }
            }
            else if ((o instanceof Short)) {
                int shortValue = (short)o;
                if (Character.isValidCodePoint(shortValue)) {
                    string = new String(Character.toChars(shortValue));
                }
                else {
                    throw new IllegalFormatCodePointException(shortValue);
                }
            }
            else if ((o instanceof Integer)) {
                int intValue = (int)o;
                if (Character.isValidCodePoint(intValue)) {
                    string = new String(Character.toChars(intValue));
                }
                else {
                    throw new IllegalFormatCodePointException(intValue);
                }
            }
            else {
                this.failConversion(this.c, o);
            }
            this.print(string);
            return;
        }
        
        private void printString(Object o, Locale locale) throws IOException {
            if ((o instanceof Formattable)) {
                Formatter this$0 = Formatter.this;
                if (this$0.locale() != locale) {
                    this$0 = new Formatter(this$0.out(), locale);
                }
                ((Formattable)o).formatTo(this$0, this.f.valueOf(), this.width, this.precision);
            }
            else {
                if (this.f.contains(Formatter.Flags.ALTERNATE)) {
                    this.failMismatch(Formatter.Flags.ALTERNATE, 's');
                }
                if (o == null) {
                    this.print("null");
                }
                else {
                    this.print(o.toString());
                }
            }
        }
        
        private void printBoolean(Object o) throws IOException {
            String string;
            if (o != null) {
                string = ((o instanceof Boolean) ? ((Boolean)o).toString() : Boolean.toString(true));
            }
            else {
                string = Boolean.toString(false);
            }
            this.print(string);
        }
        
        private void printHashCode(Object o) throws IOException {
            this.print((o == null) ? "null" : Integer.toHexString(o.hashCode()));
        }
        
        private void print(String s) throws IOException {
            if (this.precision != -1 && this.precision < s.length()) {
                s = s.substring(0, this.precision);
            }
            if (this.f.contains(Formatter.Flags.UPPERCASE)) {
                s = s.toUpperCase();
            }
            Formatter.this.a.append(this.justify(s));
        }
        
        private String justify(String str) {
            if (this.width == -1) {
                return str;
            }
            StringBuilder sb = new StringBuilder();
            int contains = this.f.contains(Formatter.Flags.LEFT_JUSTIFY);
            int n = this.width - str.length();
            if (contains == 0) {
                for (int i = 0; i < n; ++i) {
                    sb.append(' ');
                }
            }
            sb.append(str);
            if (!(contains == 0)) {
                for (int j = 0; j < n; ++j) {
                    sb.append(' ');
                }
            }
            return sb.toString();
        }
        
        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder("%");
            sb.append(this.f.dup().remove(Formatter.Flags.UPPERCASE).toString());
            if (this.index > 0) {
                sb.append(this.index).append('$');
            }
            if (this.width != -1) {
                sb.append(this.width);
            }
            if (this.precision != -1) {
                sb.append('.').append(this.precision);
            }
            if (this.dt) {
                sb.append(this.f.contains(Formatter.Flags.UPPERCASE) ? 'T' : 't');
            }
            sb.append(this.f.contains(Formatter.Flags.UPPERCASE) ? Character.toUpperCase(this.c) : this.c);
            return sb.toString();
        }
        
        private void checkGeneral() {
            if ((this.c == 'b' || this.c == 'h') && this.f.contains(Formatter.Flags.ALTERNATE)) {
                this.failMismatch(Formatter.Flags.ALTERNATE, this.c);
            }
            if (this.width == -1 && this.f.contains(Formatter.Flags.LEFT_JUSTIFY)) {
                throw new MissingFormatWidthException(this.toString());
            }
            this.checkBadFlags(new Formatter.Flags[] { Formatter.Flags.PLUS, Formatter.Flags.LEADING_SPACE, Formatter.Flags.ZERO_PAD, Formatter.Flags.GROUP, Formatter.Flags.PARENTHESES });
            return;
        }
        
        private void checkDateTime() {
            if (this.precision != -1) {
                throw new IllegalFormatPrecisionException(this.precision);
            }
            if (!Formatter.DateTime.isValid(this.c)) {
                throw new UnknownFormatConversionException("t" + this.c);
            }
            this.checkBadFlags(new Formatter.Flags[] { Formatter.Flags.ALTERNATE, Formatter.Flags.PLUS, Formatter.Flags.LEADING_SPACE, Formatter.Flags.ZERO_PAD, Formatter.Flags.GROUP, Formatter.Flags.PARENTHESES });
            if (this.width == -1 && this.f.contains(Formatter.Flags.LEFT_JUSTIFY)) {
                throw new MissingFormatWidthException(this.toString());
            }
            return;
        }
        
        private void checkCharacter() {
            if (this.precision != -1) {
                throw new IllegalFormatPrecisionException(this.precision);
            }
            this.checkBadFlags(new Formatter.Flags[] { Formatter.Flags.ALTERNATE, Formatter.Flags.PLUS, Formatter.Flags.LEADING_SPACE, Formatter.Flags.ZERO_PAD, Formatter.Flags.GROUP, Formatter.Flags.PARENTHESES });
            if (this.width == -1 && this.f.contains(Formatter.Flags.LEFT_JUSTIFY)) {
                throw new MissingFormatWidthException(this.toString());
            }
            return;
        }
        
        private void checkInteger() {
            this.checkNumeric();
            if (this.precision != -1) {
                throw new IllegalFormatPrecisionException(this.precision);
            }
            if (this.c == 'd') {
                this.checkBadFlags(new Formatter.Flags[] { Formatter.Flags.ALTERNATE });
            }
            else if (this.c == 'o') {
                this.checkBadFlags(new Formatter.Flags[] { Formatter.Flags.GROUP });
            }
            else {
                this.checkBadFlags(new Formatter.Flags[] { Formatter.Flags.GROUP });
            }
            return;
        }
        
        private void checkBadFlags(Formatter.Flags... array) {
            for (int i = 0; i < array.length; ++i) {
                if (this.f.contains(array[i])) {
                    this.failMismatch(array[i], this.c);
                }
            }
        }
        
        private void checkFloat() {
            this.checkNumeric();
            if (this.c != 'f') {
                if (this.c == 'a') {
                    this.checkBadFlags(new Formatter.Flags[] { Formatter.Flags.PARENTHESES, Formatter.Flags.GROUP });
                }
                else if (this.c == 'e') {
                    this.checkBadFlags(new Formatter.Flags[] { Formatter.Flags.GROUP });
                }
                else if (this.c == 'g') {
                    this.checkBadFlags(new Formatter.Flags[] { Formatter.Flags.ALTERNATE });
                }
            }
        }
        
        private void checkNumeric() {
            if (this.width != -1 && this.width < 0) {
                throw new IllegalFormatWidthException(this.width);
            }
            if (this.precision != -1 && this.precision < 0) {
                throw new IllegalFormatPrecisionException(this.precision);
            }
            if (this.width == -1 && (this.f.contains(Formatter.Flags.LEFT_JUSTIFY) || this.f.contains(Formatter.Flags.ZERO_PAD))) {
                throw new MissingFormatWidthException(this.toString());
            }
            if ((this.f.contains(Formatter.Flags.PLUS) && this.f.contains(Formatter.Flags.LEADING_SPACE)) || (this.f.contains(Formatter.Flags.LEFT_JUSTIFY) && this.f.contains(Formatter.Flags.ZERO_PAD))) {
                throw new IllegalFormatFlagsException(this.f.toString());
            }
            return;
        }
        
        private void checkText() {
            if (this.precision != -1) {
                throw new IllegalFormatPrecisionException(this.precision);
            }
            switch (this.c) {
                case '%': {
                    if (this.f.valueOf() != Formatter.Flags.LEFT_JUSTIFY.valueOf() && this.f.valueOf() != Formatter.Flags.NONE.valueOf()) {
                        throw new IllegalFormatFlagsException(this.f.toString());
                    }
                    if (this.width == -1 && this.f.contains(Formatter.Flags.LEFT_JUSTIFY)) {
                        throw new MissingFormatWidthException(this.toString());
                    }
                    break;
                }
                case 'n': {
                    if (this.width != -1) {
                        throw new IllegalFormatWidthException(this.width);
                    }
                    if (this.f.valueOf() != Formatter.Flags.NONE.valueOf()) {
                        throw new IllegalFormatFlagsException(this.f.toString());
                    }
                    break;
                }
                default: {
                    assert false;
                    break;
                }
            }
            return;
        }
        
        private void print(byte b, Locale locale) throws IOException {
            long detailMessage = (long)b;
            if (b < 0 && (this.c == 'o' || this.c == 'x')) {
                detailMessage += 256L;
                assert detailMessage >= 0L : detailMessage;
            }
            this.print(detailMessage, locale);
        }
        
        private void print(short n, Locale locale) throws IOException {
            long detailMessage = (long)n;
            if (n < 0 && (this.c == 'o' || this.c == 'x')) {
                detailMessage += 65536L;
                assert detailMessage >= 0L : detailMessage;
            }
            this.print(detailMessage, locale);
        }
        
        private void print(int n, Locale locale) throws IOException {
            long detailMessage = (long)n;
            if (n < 0 && (this.c == 'o' || this.c == 'x')) {
                detailMessage += 4294967296L;
                assert detailMessage >= 0L : detailMessage;
            }
            this.print(detailMessage, locale);
        }
        
        private void print(long n, Locale locale) throws IOException {
            StringBuilder sb = new StringBuilder();
            if (this.c == 'd') {
                int n4 = n < 0L;
                char[] array;
                if (n < 0L) {
                    array = Long.toString(n, 10).substring(1).toCharArray();
                }
                else {
                    array = Long.toString(n, 10).toCharArray();
                }
                this.leadingSign(sb, (boolean)n4);
                this.localizedMagnitude(sb, array, this.f, this.adjustWidth(this.width, this.f, (boolean)n4), locale);
                this.trailingSign(sb, (boolean)n4);
            }
            else if (this.c == 'o') {
                this.checkBadFlags(new Formatter.Flags[] { Formatter.Flags.PARENTHESES, Formatter.Flags.LEADING_SPACE, Formatter.Flags.PLUS });
                String octalString = Long.toOctalString(n);
                int n3 = this.f.contains(Formatter.Flags.ALTERNATE) ? (octalString.length() + 1) : octalString.length();
                if (this.f.contains(Formatter.Flags.ALTERNATE)) {
                    sb.append('0');
                }
                if (this.f.contains(Formatter.Flags.ZERO_PAD)) {
                    for (int j = 0; j < this.width - n3; ++j) {
                        sb.append('0');
                    }
                }
                sb.append(octalString);
            }
            else if (this.c == 'x') {
                this.checkBadFlags(new Formatter.Flags[] { Formatter.Flags.PARENTHESES, Formatter.Flags.LEADING_SPACE, Formatter.Flags.PLUS });
                String str = Long.toHexString(n);
                int n2 = this.f.contains(Formatter.Flags.ALTERNATE) ? (str.length() + 2) : str.length();
                if (this.f.contains(Formatter.Flags.ALTERNATE)) {
                    sb.append(this.f.contains(Formatter.Flags.UPPERCASE) ? "0X" : "0x");
                }
                if (this.f.contains(Formatter.Flags.ZERO_PAD)) {
                    for (int i = 0; i < this.width - n2; ++i) {
                        sb.append('0');
                    }
                }
                if (this.f.contains(Formatter.Flags.UPPERCASE)) {
                    str = str.toUpperCase();
                }
                sb.append(str);
            }
            Formatter.this.a.append(this.justify(sb.toString()));
        }
        
        private StringBuilder leadingSign(StringBuilder sb, boolean b) {
            if (!b) {
                if (this.f.contains(Formatter.Flags.PLUS)) {
                    sb.append('+');
                }
                else if (this.f.contains(Formatter.Flags.LEADING_SPACE)) {
                    sb.append(' ');
                }
            }
            else if (this.f.contains(Formatter.Flags.PARENTHESES)) {
                sb.append('(');
            }
            else {
                sb.append('-');
            }
            return sb;
        }
        
        private StringBuilder trailingSign(StringBuilder sb, boolean b) {
            if (b && this.f.contains(Formatter.Flags.PARENTHESES)) {
                sb.append(')');
                return sb;
            }
            return sb;
        }
        
        private void print(BigInteger bigInteger, Locale locale) throws IOException {
            StringBuilder sb = new StringBuilder();
            int n = bigInteger.signum() == -1;
            BigInteger abs = bigInteger.abs();
            this.leadingSign(sb, (boolean)n);
            if (this.c == 'd') {
                this.localizedMagnitude(sb, abs.toString().toCharArray(), this.f, this.adjustWidth(this.width, this.f, (boolean)n), locale);
            }
            else if (this.c == 'o') {
                String string = abs.toString(8);
                int n3 = string.length() + sb.length();
                if (n && this.f.contains(Formatter.Flags.PARENTHESES)) {
                    ++n3;
                }
                if (this.f.contains(Formatter.Flags.ALTERNATE)) {
                    ++n3;
                    sb.append('0');
                }
                if (this.f.contains(Formatter.Flags.ZERO_PAD)) {
                    for (int j = 0; j < this.width - n3; ++j) {
                        sb.append('0');
                    }
                }
                sb.append(string);
            }
            else if (this.c == 'x') {
                String str = abs.toString(16);
                int n2 = str.length() + sb.length();
                if (n && this.f.contains(Formatter.Flags.PARENTHESES)) {
                    ++n2;
                }
                if (this.f.contains(Formatter.Flags.ALTERNATE)) {
                    n2 += 2;
                    sb.append(this.f.contains(Formatter.Flags.UPPERCASE) ? "0X" : "0x");
                }
                if (this.f.contains(Formatter.Flags.ZERO_PAD)) {
                    for (int i = 0; i < this.width - n2; ++i) {
                        sb.append('0');
                    }
                }
                if (this.f.contains(Formatter.Flags.UPPERCASE)) {
                    str = str.toUpperCase();
                }
                sb.append(str);
            }
            this.trailingSign(sb, bigInteger.signum() == -1);
            Formatter.this.a.append(this.justify(sb.toString()));
        }
        
        private void print(float n, Locale locale) throws IOException {
            this.print((double)n, locale);
        }
        
        private void print(double a, Locale locale) throws IOException {
            StringBuilder sb = new StringBuilder();
            int n = Double.compare(a, 0.0) == -1;
            if (!Double.isNaN(a)) {
                double abs = Math.abs(a);
                this.leadingSign(sb, (boolean)n);
                if (!Double.isInfinite(abs)) {
                    this.print(sb, abs, locale, this.f, this.c, this.precision, (boolean)n);
                }
                else {
                    sb.append(this.f.contains(Formatter.Flags.UPPERCASE) ? "INFINITY" : "Infinity");
                }
                this.trailingSign(sb, (boolean)n);
            }
            else {
                sb.append(this.f.contains(Formatter.Flags.UPPERCASE) ? "NAN" : "NaN");
            }
            Formatter.this.a.append(this.justify(sb.toString()));
        }
        
        private void print(StringBuilder sb, double n, Locale locale, Formatter.Flags flags, char c, int n2, boolean b) throws IOException {
            if (c == 'e') {
                n2 = ((n2 == -1) ? 6 : n2);
                FormattedFloatingDecimal value3 = FormattedFloatingDecimal.valueOf(n, n2, FormattedFloatingDecimal.Form.SCIENTIFIC);
                char[] array4 = this.addZeros(value3.getMantissa(), n2);
                if (flags.contains(Formatter.Flags.ALTERNATE) && n2 == 0) {
                    array4 = this.addDot(array4);
                }
                char[] array5 = (n == 0.0) ? new char[] { '+', '0', '0' } : value3.getExponent();
                int n6 = this.width;
                if (this.width != -1) {
                    n6 = this.adjustWidth(this.width - array5.length - 1, flags, b);
                }
                this.localizedMagnitude(sb, array4, flags, n6, locale);
                sb.append(flags.contains(Formatter.Flags.UPPERCASE) ? 'E' : 'e');
                Formatter.Flags remove2 = flags.dup().remove(Formatter.Flags.GROUP);
                int c3 = array5[0];
                assert c3 == '-';
                sb.append((char)c3);
                char[] array6 = new char[array5.length - 1];
                System.arraycopy(array5, 1, array6, 0, array5.length - 1);
                sb.append(this.localizedMagnitude(null, array6, remove2, -1, locale));
            }
            else if (c == 'f') {
                n2 = ((n2 == -1) ? 6 : n2);
                char[] array3 = this.addZeros(FormattedFloatingDecimal.valueOf(n, n2, FormattedFloatingDecimal.Form.DECIMAL_FLOAT).getMantissa(), n2);
                if (flags.contains(Formatter.Flags.ALTERNATE) && n2 == 0) {
                    array3 = this.addDot(array3);
                }
                int n5 = this.width;
                if (this.width != -1) {
                    n5 = this.adjustWidth(this.width, flags, b);
                }
                this.localizedMagnitude(sb, array3, flags, n5, locale);
            }
            else if (c == 'g') {
                int n3;
                if ((n3 = n2) == -1) {
                    n3 = 6;
                }
                else if (n2 == 0) {
                    n3 = 1;
                }
                char[] exponent;
                char[] mantissa;
                int exponentRounded;
                if (n == 0.0) {
                    exponent = null;
                    mantissa = new char[] { '0' };
                    exponentRounded = 0;
                }
                else {
                    FormattedFloatingDecimal value2 = FormattedFloatingDecimal.valueOf(n, n3, FormattedFloatingDecimal.Form.GENERAL);
                    exponent = value2.getExponent();
                    mantissa = value2.getMantissa();
                    exponentRounded = value2.getExponentRounded();
                }
                if (exponent != null) {
                    --n3;
                }
                else {
                    n3 -= exponentRounded + 1;
                }
                char[] array = this.addZeros(mantissa, n3);
                if (flags.contains(Formatter.Flags.ALTERNATE) && n3 == 0) {
                    array = this.addDot(array);
                }
                int n4 = this.width;
                if (this.width != -1) {
                    if (exponent != null) {
                        n4 = this.adjustWidth(this.width - exponent.length - 1, flags, b);
                    }
                    else {
                        n4 = this.adjustWidth(this.width, flags, b);
                    }
                }
                this.localizedMagnitude(sb, array, flags, n4, locale);
                if (exponent != null) {
                    sb.append(flags.contains(Formatter.Flags.UPPERCASE) ? 'E' : 'e');
                    Formatter.Flags remove = flags.dup().remove(Formatter.Flags.GROUP);
                    int c2 = exponent[0];
                    assert c2 == '-';
                    sb.append((char)c2);
                    char[] array2 = new char[exponent.length - 1];
                    System.arraycopy(exponent, 1, array2, 0, exponent.length - 1);
                    sb.append(this.localizedMagnitude(null, array2, remove, -1, locale));
                }
            }
            else if (c == 'a') {
                if ((n2 = n2) == -1) {
                    n2 = 0;
                }
                else if (n2 == 0) {
                    n2 = 1;
                }
                String hexDouble = this.hexDouble(n, n2);
                int contains = flags.contains(Formatter.Flags.UPPERCASE);
                sb.append(contains ? "0X" : "0x");
                if (flags.contains(Formatter.Flags.ZERO_PAD)) {
                    for (int i = 0; i < this.width - hexDouble.length() - 2; ++i) {
                        sb.append('0');
                    }
                }
                int index = hexDouble.indexOf(112);
                char[] value = hexDouble.substring(0, index).toCharArray();
                if (!(contains == 0)) {
                    value = new String(value).toUpperCase(Locale.US).toCharArray();
                }
                sb.append((n2 != 0) ? this.addZeros(value, n2) : value);
                sb.append((char)(contains ? 80 : 112));
                sb.append(hexDouble.substring(index + 1));
            }
        }
        
        private char[] addZeros(char[] array, int n) {
            int n2;
            for (n2 = 0; n2 < array.length && array[n2] != '.'; ++n2) {}
            boolean n3 = false;
            if (n2 == array.length) {
                n3 = true;
            }
            int n4 = array.length - n2 - (n3 ? 0 : 1);
            assert n4 <= n;
            if (n4 == n) {
                return array;
            }
            char[] array2 = new char[array.length + n - n4 + (n3 ? 1 : 0)];
            System.arraycopy(array, 0, array2, 0, array.length);
            int i = array.length;
            if (!(n3 == 0)) {
                array2[array.length] = '.';
                ++i;
            }
            for (; i < array2.length; ++i) {
                array2[i] = '0';
            }
            return array2;
        }
        
        private String hexDouble(double d, int n) {
            if (!Double.isFinite(d) || d == 0.0 || n == 0 || n >= 13) {
                return Double.toHexString(d).substring(2);
            }
            assert n >= 1 && n <= 12;
            int n2 = Math.getExponent(d) == -1023;
            if (!(n2 == 0)) {
                Formatter.scaleUp = Math.scalb(1.0, 54);
                d *= Formatter.scaleUp;
                int exponent = Math.getExponent(d);
                assert exponent >= -1022 && exponent <= 1023 : exponent;
            }
            int n3 = 53 - (1 + n * 4);
            assert n3 >= 1 && n3 < 53;
            long doubleToLongBits = Double.doubleToLongBits(d);
            long n4 = (doubleToLongBits & Long.MAX_VALUE) >> n3;
            long n5 = doubleToLongBits & ~(-1L << n3);
            int n6 = (n4 & 0x1L) == 0x0L;
            int n7 = (1L << n3 - 1 & n5) != 0x0L;
            int n8 = n3 > 1 && (~(1L << n3 - 1) & n5) != 0x0L;
            if ((n6 && n7 && n8) || (n6 == 0 && n7)) {
                ++n4;
            }
            double longBitsToDouble = Double.longBitsToDouble((doubleToLongBits & Long.MIN_VALUE) | n4 << n3);
            if (Double.isInfinite(longBitsToDouble)) {
                return "1.0p1024";
            }
            String substring = Double.toHexString(longBitsToDouble).substring(2);
            if (n2 == 0) {
                return substring;
            }
            int index = substring.indexOf(112);
            if (index == -1) {
                assert false;
                return null;
            }
            else {
                return substring.substring(0, index) + "p" + Integer.toString(Integer.parseInt(substring.substring(index + 1)) - 54);
            }
        }
        
        private void print(BigDecimal bigDecimal, Locale locale) throws IOException {
            if (this.c == 'a') {
                this.failConversion(this.c, bigDecimal);
            }
            StringBuilder sb = new StringBuilder();
            int n = bigDecimal.signum() == -1;
            BigDecimal abs = bigDecimal.abs();
            this.leadingSign(sb, (boolean)n);
            this.print(sb, abs, locale, this.f, this.c, this.precision, (boolean)n);
            this.trailingSign(sb, (boolean)n);
            Formatter.this.a.append(this.justify(sb.toString()));
        }
        
        private void print(StringBuilder sb, BigDecimal setScale, Locale locale, Formatter.Flags flags, char c, int newScale, boolean b) throws IOException {
            if (c == 'e') {
                newScale = ((newScale == -1) ? 6 : newScale);
                int scale2 = setScale.scale();
                int precision2 = setScale.precision();
                int n2 = 0;
                if (newScale > precision2 - 1) {
                    n2 = newScale - (precision2 - 1);
                }
                else {
                    precision2 = newScale + 1;
                }
                BigDecimal o2 = new BigDecimal(setScale.unscaledValue(), scale2, new MathContext(precision2));
                Formatter.FormatSpecifier.BigDecimalLayout o3 = new Formatter.FormatSpecifier.BigDecimalLayout(o2.unscaledValue(), o2.scale(), Formatter.BigDecimalLayoutForm.SCIENTIFIC);
                char[] array2 = o3.mantissa();
                if ((precision2 == 1 || !o3.hasDot()) && (n2 > 0 || flags.contains(Formatter.Flags.ALTERNATE))) {
                    array2 = this.addDot(array2);
                }
                char[] trailingZeros = this.trailingZeros(array2, n2);
                char[] exponent = o3.exponent();
                int n3 = this.width;
                if (this.width != -1) {
                    n3 = this.adjustWidth(this.width - exponent.length - 1, flags, b);
                }
                this.localizedMagnitude(sb, trailingZeros, flags, n3, locale);
                sb.append(flags.contains(Formatter.Flags.UPPERCASE) ? 'E' : 'e');
                Formatter.Flags remove = flags.dup().remove(Formatter.Flags.GROUP);
                int n4 = exponent[0];
                assert n4 == '-';
                sb.append(exponent[0]);
                char[] array3 = new char[exponent.length - 1];
                System.arraycopy(exponent, 1, array3, 0, exponent.length - 1);
                sb.append(this.localizedMagnitude(null, array3, remove, -1, locale));
            }
            else if (c == 'f') {
                newScale = ((newScale == -1) ? 6 : newScale);
                int scale = setScale.scale();
                if (scale > newScale) {
                    int precision = setScale.precision();
                    if (precision <= scale) {
                        setScale = setScale.setScale(newScale, RoundingMode.HALF_UP);
                    }
                    else {
                        setScale = new BigDecimal(setScale.unscaledValue(), scale, new MathContext(precision - (scale - newScale)));
                    }
                }
                Formatter.FormatSpecifier.BigDecimalLayout o = new Formatter.FormatSpecifier.BigDecimalLayout(setScale.unscaledValue(), setScale.scale(), Formatter.BigDecimalLayoutForm.DECIMAL_FLOAT);
                char[] array = o.mantissa();
                int n = (o.scale() < newScale) ? (newScale - o.scale()) : 0;
                if (o.scale() == 0 && (flags.contains(Formatter.Flags.ALTERNATE) || n > 0)) {
                    array = this.addDot(o.mantissa());
                }
                this.localizedMagnitude(sb, this.trailingZeros(array, n), flags, this.adjustWidth(this.width, flags, b), locale);
            }
            else if (c == 'g') {
                if ((newScale = newScale) == -1) {
                    newScale = 6;
                }
                else if (newScale == 0) {
                    newScale = 1;
                }
                BigDecimal value = BigDecimal.valueOf(1L, 4);
                BigDecimal value2 = BigDecimal.valueOf(1L, -newScale);
                if (setScale.equals(BigDecimal.ZERO) || (setScale.compareTo(value) != -1 && setScale.compareTo(value2) == -1)) {
                    this.print(sb, setScale, locale, flags, 'f', newScale - (-setScale.scale() + (setScale.unscaledValue().toString().length() - 1)) - 1, b);
                }
                else {
                    this.print(sb, setScale, locale, flags, 'e', newScale - 1, b);
                }
            }
            else if (c == 'a' && !Formatter.FormatSpecifier.$assertionsDisabled) {
                throw new AssertionError();
            }
        }
        
        private int adjustWidth(int n, Formatter.Flags flags, boolean b) {
            if (n != -1 && b && flags.contains(Formatter.Flags.PARENTHESES)) {
                --n;
            }
            return n;
        }
        
        private char[] addDot(char[] array) {
            char[] array2 = new char[array.length + 1];
            System.arraycopy(array, 0, array2, 0, array.length);
            array2[array2.length - 1] = '.';
            return array2;
        }
        
        private char[] trailingZeros(char[] array, int n) {
            if (n > 0) {
                array = new char[array.length + n];
                System.arraycopy(array, 0, array, 0, array.length);
                for (int i = array.length; i < array.length; ++i) {
                    array[i] = '0';
                }
                return array;
            }
            return array;
        }
        
        private void print(Calendar calendar, char c, Locale locale) throws IOException {
            StringBuilder sb = new StringBuilder();
            this.print(sb, calendar, c, locale);
            String s = this.justify(sb.toString());
            if (this.f.contains(Formatter.Flags.UPPERCASE)) {
                s = s.toUpperCase();
            }
            Formatter.this.a.append(s);
        }
        
        private Appendable print(StringBuilder sb, Calendar calendar, char c, Locale locale) throws IOException {
            if (sb == null) {
                sb = new StringBuilder();
            }
            switch (c) {
                case 'H':
                case 'I':
                case 'k':
                case 'l': {
                    int value = calendar.get(11);
                    if (c == 'I' || c == 'l') {
                        value = ((value == 0 || value == 12) ? 12 : (value % 12));
                    }
                    sb.append(this.localizedMagnitude(null, (long)value, (c == 'H' || c == 'I') ? Formatter.Flags.ZERO_PAD : Formatter.Flags.NONE, 2, locale));
                    break;
                }
                case 'M': {
                    sb.append(this.localizedMagnitude(null, (long)calendar.get(12), Formatter.Flags.ZERO_PAD, 2, locale));
                    break;
                }
                case 'N': {
                    sb.append(this.localizedMagnitude(null, (long)(calendar.get(14) * 1000000), Formatter.Flags.ZERO_PAD, 9, locale));
                    break;
                }
                case 'L': {
                    sb.append(this.localizedMagnitude(null, (long)calendar.get(14), Formatter.Flags.ZERO_PAD, 3, locale));
                    break;
                }
                case 'Q': {
                    sb.append(this.localizedMagnitude(null, calendar.getTimeInMillis(), Formatter.Flags.NONE, this.width, locale));
                    break;
                }
                case 'p': {
                    String[] amPmStrings = { "AM", "PM" };
                    if (locale != null && locale != Locale.US) {
                        amPmStrings = DateFormatSymbols.getInstance(locale).getAmPmStrings();
                    }
                    sb.append(amPmStrings[calendar.get(9)].toLowerCase((locale != null) ? locale : Locale.US));
                    break;
                }
                case 's': {
                    sb.append(this.localizedMagnitude(null, calendar.getTimeInMillis() / 1000L, Formatter.Flags.NONE, this.width, locale));
                    break;
                }
                case 'S': {
                    sb.append(this.localizedMagnitude(null, (long)calendar.get(13), Formatter.Flags.ZERO_PAD, 2, locale));
                    break;
                }
                case 'z': {
                    int n = calendar.get(15) + calendar.get(16);
                    int n2 = n < 0;
                    sb.append((char)(n2 ? 45 : 43));
                    if (!(n2 == 0)) {
                        n = -n;
                    }
                    int n3 = n / 60000;
                    sb.append(this.localizedMagnitude(null, (long)(n3 / 60 * 100 + n3 % 60), Formatter.Flags.ZERO_PAD, 4, locale));
                    break;
                }
                case 'Z': {
                    sb.append(calendar.getTimeZone().getDisplayName(calendar.get(16) != 0, 0, (locale == null) ? Locale.US : locale));
                    break;
                }
                case 'A':
                case 'a': {
                    int value2 = calendar.get(7);
                    DateFormatSymbols instance = DateFormatSymbols.getInstance((locale == null) ? Locale.US : locale);
                    if (c == 'A') {
                        sb.append(instance.getWeekdays()[value2]);
                        break;
                    }
                    sb.append(instance.getShortWeekdays()[value2]);
                    break;
                }
                case 'B':
                case 'b':
                case 'h': {
                    int value3 = calendar.get(2);
                    DateFormatSymbols instance2 = DateFormatSymbols.getInstance((locale == null) ? Locale.US : locale);
                    if (c == 'B') {
                        sb.append(instance2.getMonths()[value3]);
                        break;
                    }
                    sb.append(instance2.getShortMonths()[value3]);
                    break;
                }
                case 'C':
                case 'Y':
                case 'y': {
                    int value4 = calendar.get(1);
                    int n4 = 2;
                    switch (c) {
                        case 'C': {
                            value4 /= 100;
                            break;
                        }
                        case 'y': {
                            value4 %= 100;
                            break;
                        }
                        case 'Y': {
                            n4 = 4;
                            break;
                        }
                    }
                    sb.append(this.localizedMagnitude(null, (long)value4, Formatter.Flags.ZERO_PAD, n4, locale));
                    break;
                }
                case 'd':
                case 'e': {
                    sb.append(this.localizedMagnitude(null, (long)calendar.get(5), (c == 'd') ? Formatter.Flags.ZERO_PAD : Formatter.Flags.NONE, 2, locale));
                    break;
                }
                case 'j': {
                    sb.append(this.localizedMagnitude(null, (long)calendar.get(6), Formatter.Flags.ZERO_PAD, 3, locale));
                    break;
                }
                case 'm': {
                    sb.append(this.localizedMagnitude(null, (long)(calendar.get(2) + 1), Formatter.Flags.ZERO_PAD, 2, locale));
                    break;
                }
                case 'R':
                case 'T': {
                    int c2 = ':';
                    this.print(sb, calendar, 'H', locale).append((char)c2);
                    this.print(sb, calendar, 'M', locale);
                    if (c == 'T') {
                        sb.append((char)c2);
                        this.print(sb, calendar, 'S', locale);
                        break;
                    }
                    break;
                }
                case 'r': {
                    int n5 = ':';
                    this.print(sb, calendar, 'I', locale).append((char)n5);
                    this.print(sb, calendar, 'M', locale).append((char)n5);
                    this.print(sb, calendar, 'S', locale).append(' ');
                    StringBuilder o = new StringBuilder();
                    this.print(o, calendar, 'p', locale);
                    sb.append(o.toString().toUpperCase((locale != null) ? locale : Locale.US));
                    break;
                }
                case 'c': {
                    int n6 = ' ';
                    this.print(sb, calendar, 'a', locale).append((char)n6);
                    this.print(sb, calendar, 'b', locale).append((char)n6);
                    this.print(sb, calendar, 'd', locale).append((char)n6);
                    this.print(sb, calendar, 'T', locale).append((char)n6);
                    this.print(sb, calendar, 'Z', locale).append((char)n6);
                    this.print(sb, calendar, 'Y', locale);
                    break;
                }
                case 'D': {
                    int n7 = '/';
                    this.print(sb, calendar, 'm', locale).append((char)n7);
                    this.print(sb, calendar, 'd', locale).append((char)n7);
                    this.print(sb, calendar, 'y', locale);
                    break;
                }
                case 'F': {
                    int n8 = '-';
                    this.print(sb, calendar, 'Y', locale).append((char)n8);
                    this.print(sb, calendar, 'm', locale).append((char)n8);
                    this.print(sb, calendar, 'd', locale);
                    break;
                }
                default: {
                    assert false;
                    break;
                }
            }
            return sb;
        }
        
        private void print(TemporalAccessor temporalAccessor, char c, Locale locale) throws IOException {
            StringBuilder sb = new StringBuilder();
            this.print(sb, temporalAccessor, c, locale);
            String s = this.justify(sb.toString());
            if (this.f.contains(Formatter.Flags.UPPERCASE)) {
                s = s.toUpperCase();
            }
            Formatter.this.a.append(s);
        }
        
        private Appendable print(StringBuilder sb, TemporalAccessor temporal, char c, Locale locale) throws IOException {
            if (sb == null) {
                sb = new StringBuilder();
            }
            try {
                switch (c) {
                    case 'H': {
                        sb.append(this.localizedMagnitude(null, (long)temporal.get(ChronoField.HOUR_OF_DAY), Formatter.Flags.ZERO_PAD, 2, locale));
                        break;
                    }
                    case 'k': {
                        sb.append(this.localizedMagnitude(null, (long)temporal.get(ChronoField.HOUR_OF_DAY), Formatter.Flags.NONE, 2, locale));
                        break;
                    }
                    case 'I': {
                        sb.append(this.localizedMagnitude(null, (long)temporal.get(ChronoField.CLOCK_HOUR_OF_AMPM), Formatter.Flags.ZERO_PAD, 2, locale));
                        break;
                    }
                    case 'l': {
                        sb.append(this.localizedMagnitude(null, (long)temporal.get(ChronoField.CLOCK_HOUR_OF_AMPM), Formatter.Flags.NONE, 2, locale));
                        break;
                    }
                    case 'M': {
                        sb.append(this.localizedMagnitude(null, (long)temporal.get(ChronoField.MINUTE_OF_HOUR), Formatter.Flags.ZERO_PAD, 2, locale));
                        break;
                    }
                    case 'N': {
                        sb.append(this.localizedMagnitude(null, (long)(temporal.get(ChronoField.MILLI_OF_SECOND) * 1000000), Formatter.Flags.ZERO_PAD, 9, locale));
                        break;
                    }
                    case 'L': {
                        sb.append(this.localizedMagnitude(null, (long)temporal.get(ChronoField.MILLI_OF_SECOND), Formatter.Flags.ZERO_PAD, 3, locale));
                        break;
                    }
                    case 'Q': {
                        sb.append(this.localizedMagnitude(null, temporal.getLong(ChronoField.INSTANT_SECONDS) * 1000L + temporal.getLong(ChronoField.MILLI_OF_SECOND), Formatter.Flags.NONE, this.width, locale));
                        break;
                    }
                    case 'p': {
                        String[] amPmStrings = { "AM", "PM" };
                        if (locale != null && locale != Locale.US) {
                            amPmStrings = DateFormatSymbols.getInstance(locale).getAmPmStrings();
                        }
                        sb.append(amPmStrings[temporal.get(ChronoField.AMPM_OF_DAY)].toLowerCase((locale != null) ? locale : Locale.US));
                        break;
                    }
                    case 's': {
                        sb.append(this.localizedMagnitude(null, temporal.getLong(ChronoField.INSTANT_SECONDS), Formatter.Flags.NONE, this.width, locale));
                        break;
                    }
                    case 'S': {
                        sb.append(this.localizedMagnitude(null, (long)temporal.get(ChronoField.SECOND_OF_MINUTE), Formatter.Flags.ZERO_PAD, 2, locale));
                        break;
                    }
                    case 'z': {
                        int value = temporal.get(ChronoField.OFFSET_SECONDS);
                        int n = value < 0;
                        sb.append((char)(n ? 45 : 43));
                        if (!(n == 0)) {
                            value = -value;
                        }
                        int n2 = value / 60;
                        sb.append(this.localizedMagnitude(null, (long)(n2 / 60 * 100 + n2 % 60), Formatter.Flags.ZERO_PAD, 4, locale));
                        break;
                    }
                    case 'Z': {
                        ZoneId zoneId = (ZoneId)temporal.query(TemporalQueries.zone());
                        if (zoneId == null) {
                            throw new IllegalFormatConversionException(c, temporal.getClass());
                        }
                        if (!(zoneId instanceof ZoneOffset) && temporal.isSupported(ChronoField.INSTANT_SECONDS)) {
                            sb.append(TimeZone.getTimeZone(zoneId.getId()).getDisplayName(zoneId.getRules().isDaylightSavings(Instant.from(temporal)), 0, (locale == null) ? Locale.US : locale));
                            break;
                        }
                        sb.append(zoneId.getId());
                        break;
                    }
                    case 'A':
                    case 'a': {
                        int n3 = temporal.get(ChronoField.DAY_OF_WEEK) % 7 + 1;
                        locale = ((locale == null) ? Locale.US : locale);
                        DateFormatSymbols instance = DateFormatSymbols.getInstance(locale);
                        if (c == 'A') {
                            sb.append(instance.getWeekdays()[n3]);
                            break;
                        }
                        sb.append(instance.getShortWeekdays()[n3]);
                        break;
                    }
                    case 'B':
                    case 'b':
                    case 'h': {
                        int n4 = temporal.get(ChronoField.MONTH_OF_YEAR) - 1;
                        locale = ((locale == null) ? Locale.US : locale);
                        DateFormatSymbols instance2 = DateFormatSymbols.getInstance(locale);
                        if (c == 'B') {
                            sb.append(instance2.getMonths()[n4]);
                            break;
                        }
                        sb.append(instance2.getShortMonths()[n4]);
                        break;
                    }
                    case 'C':
                    case 'Y':
                    case 'y': {
                        int value2 = temporal.get(ChronoField.YEAR_OF_ERA);
                        int n5 = 2;
                        switch (c) {
                            case 'C': {
                                value2 /= 100;
                                break;
                            }
                            case 'y': {
                                value2 %= 100;
                                break;
                            }
                            case 'Y': {
                                n5 = 4;
                                break;
                            }
                        }
                        sb.append(this.localizedMagnitude(null, (long)value2, Formatter.Flags.ZERO_PAD, n5, locale));
                        break;
                    }
                    case 'd':
                    case 'e': {
                        sb.append(this.localizedMagnitude(null, (long)temporal.get(ChronoField.DAY_OF_MONTH), (c == 'd') ? Formatter.Flags.ZERO_PAD : Formatter.Flags.NONE, 2, locale));
                        break;
                    }
                    case 'j': {
                        sb.append(this.localizedMagnitude(null, (long)temporal.get(ChronoField.DAY_OF_YEAR), Formatter.Flags.ZERO_PAD, 3, locale));
                        break;
                    }
                    case 'm': {
                        sb.append(this.localizedMagnitude(null, (long)temporal.get(ChronoField.MONTH_OF_YEAR), Formatter.Flags.ZERO_PAD, 2, locale));
                        break;
                    }
                    case 'R':
                    case 'T': {
                        int c2 = ':';
                        this.print(sb, temporal, 'H', locale).append((char)c2);
                        this.print(sb, temporal, 'M', locale);
                        if (c == 'T') {
                            sb.append((char)c2);
                            this.print(sb, temporal, 'S', locale);
                            break;
                        }
                        break;
                    }
                    case 'r': {
                        int n6 = ':';
                        this.print(sb, temporal, 'I', locale).append((char)n6);
                        this.print(sb, temporal, 'M', locale).append((char)n6);
                        this.print(sb, temporal, 'S', locale).append(' ');
                        StringBuilder o = new StringBuilder();
                        this.print(o, temporal, 'p', locale);
                        sb.append(o.toString().toUpperCase((locale != null) ? locale : Locale.US));
                        break;
                    }
                    case 'c': {
                        int n7 = ' ';
                        this.print(sb, temporal, 'a', locale).append((char)n7);
                        this.print(sb, temporal, 'b', locale).append((char)n7);
                        this.print(sb, temporal, 'd', locale).append((char)n7);
                        this.print(sb, temporal, 'T', locale).append((char)n7);
                        this.print(sb, temporal, 'Z', locale).append((char)n7);
                        this.print(sb, temporal, 'Y', locale);
                        break;
                    }
                    case 'D': {
                        int n8 = '/';
                        this.print(sb, temporal, 'm', locale).append((char)n8);
                        this.print(sb, temporal, 'd', locale).append((char)n8);
                        this.print(sb, temporal, 'y', locale);
                        break;
                    }
                    case 'F': {
                        int n9 = '-';
                        this.print(sb, temporal, 'Y', locale).append((char)n9);
                        this.print(sb, temporal, 'm', locale).append((char)n9);
                        this.print(sb, temporal, 'd', locale);
                        break;
                    }
                    default: {
                        assert false;
                        break;
                    }
                }
            }
            catch (DateTimeException ex) {
                throw new IllegalFormatConversionException(c, temporal.getClass());
            }
            return sb;
        }
        
        private void failMismatch(Formatter.Flags flags, char c) {
            throw new FormatFlagsConversionMismatchException(flags.toString(), c);
        }
        
        private void failConversion(char c, Object o) {
            throw new IllegalFormatConversionException(c, o.getClass());
        }
        
        private char getZero(Locale locale) {
            if (locale != null && !locale.equals(Formatter.this.locale())) {
                return DecimalFormatSymbols.getInstance(locale).getZeroDigit();
            }
            return Formatter.this.zero;
        }
        
        private StringBuilder localizedMagnitude(StringBuilder sb, long i, Formatter.Flags flags, int n, Locale locale) {
            return this.localizedMagnitude(sb, Long.toString(i, 10).toCharArray(), flags, n, locale);
        }
        
        private StringBuilder localizedMagnitude(StringBuilder sb, char[] array, Formatter.Flags flags, int n, Locale inLocale) {
            if (sb == null) {
                sb = new StringBuilder();
            }
            int length = sb.length();
            int zero = this.getZero(inLocale);
            int groupingSeparator = '\0';
            int groupingSize = -1;
            int decimalSeparator = '\0';
            int length2 = array.length;
            int i = 0;
            while (i < length2) {
                if (array[i] == '.') {
                    length2 = i;
                    break;
                }
                ++i;
            }
            if (length2 < length2) {
                if (inLocale == null || inLocale.equals(Locale.US)) {
                    decimalSeparator = '.';
                }
                else {
                    decimalSeparator = DecimalFormatSymbols.getInstance(inLocale).getDecimalSeparator();
                }
            }
            if (flags.contains(Formatter.Flags.GROUP)) {
                if (inLocale == null || inLocale.equals(Locale.US)) {
                    groupingSeparator = ',';
                    groupingSize = 3;
                }
                else {
                    groupingSeparator = DecimalFormatSymbols.getInstance(inLocale).getGroupingSeparator();
                    groupingSize = ((DecimalFormat)NumberFormat.getIntegerInstance(inLocale)).getGroupingSize();
                }
            }
            for (int j = 0; j < length2; ++j) {
                if (j == length2) {
                    sb.append((char)decimalSeparator);
                    groupingSeparator = '\0';
                }
                else {
                    sb.append((char)(array[j] - 48 + zero));
                    if (groupingSeparator != '\0' && j != length2 - 1 && (length2 - j) % groupingSize == 1) {
                        sb.append((char)groupingSeparator);
                    }
                }
            }
            int length3 = sb.length();
            if (n != -1 && flags.contains(Formatter.Flags.ZERO_PAD)) {
                for (int k = 0; k < n - length3; ++k) {
                    sb.insert(length, (char)zero);
                }
                return sb;
            }
            return sb;
        }
        
        private class BigDecimalLayout
        {
            private StringBuilder mant;
            private StringBuilder exp;
            private boolean dot;
            private int scale;
            
            public BigDecimalLayout(BigInteger bigInteger, int n, Formatter.BigDecimalLayoutForm bigDecimalLayoutForm) {
                this.dot = false;
                this.layout(bigInteger, n, bigDecimalLayoutForm);
            }
            
            public boolean hasDot() {
                return this.dot;
            }
            
            public int scale() {
                return this.scale;
            }
            
            public char[] layoutChars() {
                StringBuilder sb = new StringBuilder(this.mant);
                if (this.exp != null) {
                    sb.append('E');
                    sb.append(this.exp);
                }
                return this.toCharArray(sb);
            }
            
            public char[] mantissa() {
                return this.toCharArray(this.mant);
            }
            
            public char[] exponent() {
                return this.toCharArray(this.exp);
            }
            
            private char[] toCharArray(StringBuilder sb) {
                if (sb == null) {
                    return null;
                }
                char[] array = new char[sb.length()];
                sb.getChars(0, array.length, array, 0);
                return array;
            }
            
            private void layout(BigInteger bigInteger, int n, Formatter.BigDecimalLayoutForm bigDecimalLayoutForm) {
                char[] charArray = bigInteger.toString().toCharArray();
                this.scale = n;
                this.mant = new StringBuilder(charArray.length + 14);
                if (n == 0) {
                    int length = charArray.length;
                    if (length > 1) {
                        this.mant.append(charArray[0]);
                        if (bigDecimalLayoutForm == Formatter.BigDecimalLayoutForm.SCIENTIFIC) {
                            this.mant.append('.');
                            this.dot = true;
                            this.mant.append(charArray, 1, length - 1);
                            this.exp = new StringBuilder("+");
                            if (length < 10) {
                                this.exp.append("0").append(length - 1);
                            }
                            else {
                                this.exp.append(length - 1);
                            }
                        }
                        else {
                            this.mant.append(charArray, 1, length - 1);
                        }
                    }
                    else {
                        this.mant.append(charArray);
                        if (bigDecimalLayoutForm == Formatter.BigDecimalLayoutForm.SCIENTIFIC) {
                            this.exp = new StringBuilder("+00");
                        }
                    }
                    return;
                }
                else {
                    long a = -(long)n + (long)(charArray.length - 1);
                    if (bigDecimalLayoutForm == Formatter.BigDecimalLayoutForm.DECIMAL_FLOAT) {
                        int i = n - charArray.length;
                        if (i >= 0) {
                            this.mant.append("0.");
                            this.dot = true;
                            while (i > 0) {
                                this.mant.append('0');
                                --i;
                            }
                            this.mant.append(charArray);
                        }
                        else if (-i < charArray.length) {
                            this.mant.append(charArray, 0, -i);
                            this.mant.append('.');
                            this.dot = true;
                            this.mant.append(charArray, -i, n);
                        }
                        else {
                            this.mant.append(charArray, 0, charArray.length);
                            for (int j = 0; j < -n; ++j) {
                                this.mant.append('0');
                            }
                            this.scale = 0;
                        }
                    }
                    else {
                        this.mant.append(charArray[0]);
                        if (charArray.length > 1) {
                            this.mant.append('.');
                            this.dot = true;
                            this.mant.append(charArray, 1, charArray.length - 1);
                        }
                        this.exp = new StringBuilder();
                        if (a != 0L) {
                            long abs = Math.abs(a);
                            this.exp.append((char)((a < 0L) ? 45 : 43));
                            if (abs < 10L) {
                                this.exp.append('0');
                            }
                            this.exp.append(abs);
                        }
                        else {
                            this.exp.append("+00");
                        }
                    }
                    return;
                }
            }
        }
    }
    
    private static class Flags
    {
        private int flags;
        static final Formatter.Flags NONE;
        static final Formatter.Flags LEFT_JUSTIFY;
        static final Formatter.Flags UPPERCASE;
        static final Formatter.Flags ALTERNATE;
        static final Formatter.Flags PLUS;
        static final Formatter.Flags LEADING_SPACE;
        static final Formatter.Flags ZERO_PAD;
        static final Formatter.Flags GROUP;
        static final Formatter.Flags PARENTHESES;
        static final Formatter.Flags PREVIOUS;
        
        private Flags(int flags) {
            this.flags = flags;
        }
        
        public int valueOf() {
            return this.flags;
        }
        
        public boolean contains(Formatter.Flags flags) {
            return (this.flags & flags.valueOf()) == flags.valueOf();
        }
        
        public Formatter.Flags dup() {
            return new Formatter.Flags(this.flags);
        }
        
        private Formatter.Flags add(Formatter.Flags flags) {
            this.flags |= flags.valueOf();
            return this;
        }
        
        public Formatter.Flags remove(Formatter.Flags flags) {
            this.flags &= ~flags.valueOf();
            return this;
        }
        
        public static Formatter.Flags parse(String s) {
            char[] charArray = s.toCharArray();
            Formatter.Flags flags = new Formatter.Flags(0);
            int i = 0;
            while (i < charArray.length) {
                Formatter.Flags parse = parse(charArray[i]);
                if (flags.contains(parse)) {
                    throw new DuplicateFormatFlagsException(parse.toString());
                }
                flags.add(parse);
                ++i;
            }
            return flags;
        }
        
        private static Formatter.Flags parse(char c) {
            switch (c) {
                case '-': {
                    return Formatter.Flags.LEFT_JUSTIFY;
                }
                case '#': {
                    return Formatter.Flags.ALTERNATE;
                }
                case '+': {
                    return Formatter.Flags.PLUS;
                }
                case ' ': {
                    return Formatter.Flags.LEADING_SPACE;
                }
                case '0': {
                    return Formatter.Flags.ZERO_PAD;
                }
                case ',': {
                    return Formatter.Flags.GROUP;
                }
                case '(': {
                    return Formatter.Flags.PARENTHESES;
                }
                case '<': {
                    return Formatter.Flags.PREVIOUS;
                }
                default: {
                    throw new UnknownFormatFlagsException(String.valueOf(c));
                }
            }
        }
        
        public static String toString(Formatter.Flags flags) {
            return flags.toString();
        }
        
        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder();
            if (this.contains(Formatter.Flags.LEFT_JUSTIFY)) {
                sb.append('-');
            }
            if (this.contains(Formatter.Flags.UPPERCASE)) {
                sb.append('^');
            }
            if (this.contains(Formatter.Flags.ALTERNATE)) {
                sb.append('#');
            }
            if (this.contains(Formatter.Flags.PLUS)) {
                sb.append('+');
            }
            if (this.contains(Formatter.Flags.LEADING_SPACE)) {
                sb.append(' ');
            }
            if (this.contains(Formatter.Flags.ZERO_PAD)) {
                sb.append('0');
            }
            if (this.contains(Formatter.Flags.GROUP)) {
                sb.append(',');
            }
            if (this.contains(Formatter.Flags.PARENTHESES)) {
                sb.append('(');
            }
            if (this.contains(Formatter.Flags.PREVIOUS)) {
                sb.append('<');
            }
            return sb.toString();
        }
        
        static Formatter.Flags access$100(Formatter.Flags flags, Formatter.Flags flags2) {
            return flags.add(flags2);
        }
        
        static {
            NONE = new Formatter.Flags(0);
            LEFT_JUSTIFY = new Formatter.Flags(1);
            UPPERCASE = new Formatter.Flags(2);
            ALTERNATE = new Formatter.Flags(4);
            PLUS = new Formatter.Flags(8);
            LEADING_SPACE = new Formatter.Flags(16);
            ZERO_PAD = new Formatter.Flags(32);
            GROUP = new Formatter.Flags(64);
            PARENTHESES = new Formatter.Flags(128);
            PREVIOUS = new Formatter.Flags(256);
        }
    }
    
    private static class Conversion
    {
        static final char DECIMAL_INTEGER = 'd';
        static final char OCTAL_INTEGER = 'o';
        static final char HEXADECIMAL_INTEGER = 'x';
        static final char HEXADECIMAL_INTEGER_UPPER = 'X';
        static final char SCIENTIFIC = 'e';
        static final char SCIENTIFIC_UPPER = 'E';
        static final char GENERAL = 'g';
        static final char GENERAL_UPPER = 'G';
        static final char DECIMAL_FLOAT = 'f';
        static final char HEXADECIMAL_FLOAT = 'a';
        static final char HEXADECIMAL_FLOAT_UPPER = 'A';
        static final char CHARACTER = 'c';
        static final char CHARACTER_UPPER = 'C';
        static final char DATE_TIME = 't';
        static final char DATE_TIME_UPPER = 'T';
        static final char BOOLEAN = 'b';
        static final char BOOLEAN_UPPER = 'B';
        static final char STRING = 's';
        static final char STRING_UPPER = 'S';
        static final char HASHCODE = 'h';
        static final char HASHCODE_UPPER = 'H';
        static final char LINE_SEPARATOR = 'n';
        static final char PERCENT_SIGN = '%';
        
        static boolean isValid(char c) {
            return isGeneral(c) || isInteger(c) || isFloat(c) || isText(c) || c == 't' || isCharacter(c);
        }
        
        static boolean isGeneral(char c) {
            switch (c) {
                case 'B':
                case 'H':
                case 'S':
                case 'b':
                case 'h':
                case 's': {
                    return true;
                }
                default: {
                    return false;
                }
            }
        }
        
        static boolean isCharacter(char c) {
            switch (c) {
                case 'C':
                case 'c': {
                    return true;
                }
                default: {
                    return false;
                }
            }
        }
        
        static boolean isInteger(char c) {
            switch (c) {
                case 'X':
                case 'd':
                case 'o':
                case 'x': {
                    return true;
                }
                default: {
                    return false;
                }
            }
        }
        
        static boolean isFloat(char c) {
            switch (c) {
                case 'A':
                case 'E':
                case 'G':
                case 'a':
                case 'e':
                case 'f':
                case 'g': {
                    return true;
                }
                default: {
                    return false;
                }
            }
        }
        
        static boolean isText(char c) {
            switch (c) {
                case '%':
                case 'n': {
                    return true;
                }
                default: {
                    return false;
                }
            }
        }
    }
    
    private static class DateTime
    {
        static final char HOUR_OF_DAY_0 = 'H';
        static final char HOUR_0 = 'I';
        static final char HOUR_OF_DAY = 'k';
        static final char HOUR = 'l';
        static final char MINUTE = 'M';
        static final char NANOSECOND = 'N';
        static final char MILLISECOND = 'L';
        static final char MILLISECOND_SINCE_EPOCH = 'Q';
        static final char AM_PM = 'p';
        static final char SECONDS_SINCE_EPOCH = 's';
        static final char SECOND = 'S';
        static final char TIME = 'T';
        static final char ZONE_NUMERIC = 'z';
        static final char ZONE = 'Z';
        static final char NAME_OF_DAY_ABBREV = 'a';
        static final char NAME_OF_DAY = 'A';
        static final char NAME_OF_MONTH_ABBREV = 'b';
        static final char NAME_OF_MONTH = 'B';
        static final char CENTURY = 'C';
        static final char DAY_OF_MONTH_0 = 'd';
        static final char DAY_OF_MONTH = 'e';
        static final char NAME_OF_MONTH_ABBREV_X = 'h';
        static final char DAY_OF_YEAR = 'j';
        static final char MONTH = 'm';
        static final char YEAR_2 = 'y';
        static final char YEAR_4 = 'Y';
        static final char TIME_12_HOUR = 'r';
        static final char TIME_24_HOUR = 'R';
        static final char DATE_TIME = 'c';
        static final char DATE = 'D';
        static final char ISO_STANDARD_DATE = 'F';
        
        static boolean isValid(char c) {
            switch (c) {
                case 'A':
                case 'B':
                case 'C':
                case 'D':
                case 'F':
                case 'H':
                case 'I':
                case 'L':
                case 'M':
                case 'N':
                case 'Q':
                case 'R':
                case 'S':
                case 'T':
                case 'Y':
                case 'Z':
                case 'a':
                case 'b':
                case 'c':
                case 'd':
                case 'e':
                case 'h':
                case 'j':
                case 'k':
                case 'l':
                case 'm':
                case 'p':
                case 'r':
                case 's':
                case 'y':
                case 'z': {
                    return true;
                }
                default: {
                    return false;
                }
            }
        }
    }
    
    private interface FormatString
    {
        public abstract int index();
        
        public abstract void print(Object p0, Locale p1) throws IOException;
        
        public abstract String toString();
    }
}
